# Copyright (C) 2009 Chris Larson <clarson@kergoth.com>
# Released under the MIT license (see COPYING.MIT for the terms)
#
# gitver.bbclass provides a GITVER variable which is a (fairly) sane version,
# for use in ${PV}, extracted from the ${S} git checkout, assuming it is one.
# This is most useful in concert with srctree.bbclass.

def git_drop_tag_prefix(version):
    import re
    if re.match("v\d", version):
        return version[1:]
    else:
        return version

# Assume that the Git repository is in ${S} unless the recipe tells us otherwise
GITVER_S ?= "${S}"

GIT_TAGADJUST = "git_drop_tag_prefix(version)"
GITVER = "${@get_git_pv('${GITVER_S}', d, match=None, tagadjust=lambda version:${GIT_TAGADJUST})}"
GITSHA = "${@get_git_hash('${GITVER_S}', d)}"

TAG_MACHINE = "${MACHINE}"
# If we're building the tests, get the tag from the underlying machine instead
TAG_MACHINE_cheetah-tests = "cheetah"
TAG_MACHINE_panther-tests = "panther"
TAG_MACHINE_qemux86 = "panther"

BS_GITVER = "${@get_git_pv('${GITVER_S}', d, match='release/${TAG_MACHINE}/*', tagadjust=brightsign_tagadjust)}"

def brightsign_tagadjust(version):
    import re
    return re.sub(r'^([^/]*/)*', '', version)

def get_git_hash(path, d):
    return oe_run(d, ["/usr/bin/git", "rev-parse", "--short", "HEAD"], cwd=path).rstrip()

def get_git_pv(path, d, match=None, tagadjust=None):
    import os
    import oe.process

    rootdir = os.path.abspath(d.getVar("GITVER_S", True))
    try:
	args = [ "/usr/bin/git", "describe", "--tags" ]
	if match:
	   args.extend(["--match", match])

        ver = oe_run(d, args, cwd=rootdir).rstrip()
    except Exception, exc:
        # Don't fail here during recipe parsing
        ver = "0_not_in_git_"

    if not ver:
        try:
            ver = get_git_hash(rootdir, d)
        except Exception, exc:
            bb.fatal(str(exc))

        if ver:
            return "0.0+%s" % ver
        else:
            return "0.0"
    else:
        if tagadjust:
            ver = tagadjust(ver)
        return ver

def mark_recipe_dependencies(path, d):
    import re
    from bb.parse import mark_dependency

    gitdir = os.path.join(path, ".git")

    # If the .git directory is actually somewhere else let git find it for us.
    try:
	new_gitdir = oe_run(d, ["/usr/bin/git", "rev-parse", "--resolve-git-dir", gitdir ]).rstrip()
	if not re.match("^--resolve-git-dir\n", new_gitdir, re.MULTILINE):
	   gitdir = new_gitdir
    except Exception, exc:
        bb.fatal(str(exc))

    # Force the recipe to be reparsed so the version gets bumped
    # if the active branch is switched, or if the branch changes.
    mark_dependency(d, os.path.join(gitdir, "HEAD"))

    # Force a reparse if anything in the index changes.
    mark_dependency(d, os.path.join(gitdir, "index"))

    # Force a reparse if any new tags of the form we're looking for
    # appear. There might not be any (they might all be packed) so
    # make sure we depend on the first directory that does exist so
    # that if the lower ones are created we'll be parsed again.
    if os.path.isdir(os.path.join(gitdir, "refs/tags")):
        if os.path.isdir(os.path.join(gitdir, "refs/tags/release")):
            if os.path.isdir(os.path.join(gitdir, "refs/tags/release", d.getVar("MACHINE", True))):
                mark_dependency(d, os.path.join(gitdir, "refs/tags/release", d.getVar("MACHINE", True)))
            else:
                mark_dependency(d, os.path.join(gitdir, "refs/tags/release"))
        else:
            mark_dependency(d, os.path.join(gitdir, "refs/tags"))

    # Just in case they've been packed we'd better look at the
    # packed-refs too
    if os.path.isfile(os.path.join(gitdir, "packed-refs")):
        mark_dependency(d, os.path.join(gitdir, "packed-refs"))
    
    try:
        ref = oe_run(d, ["/usr/bin/git", "symbolic-ref", "-q", "HEAD"], cwd=gitdir).rstrip()
    except oe.process.CmdError:
        pass
    else:
        # Even this ref might be a packed one so only add it if the file exists
        if ref and os.path.isfile(os.path.join(gitdir, ref)):
            mark_dependency(d, os.path.join(gitdir, ref))

    # Catch new tags.
    tagdir = os.path.join(gitdir, "refs", "tags")
    if os.path.exists(tagdir):
        mark_dependency(d, tagdir)

python () {
    mark_recipe_dependencies(d.getVar("GITVER_S", True), d)
}
